"""
Classifies: CHEBI:25036 lignan
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors
from rdkit.Chem.Draw import IPythonConsole

def is_lignan(smiles: str):
    """
    Determines if a molecule is a lignan based on structural characteristics.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a lignan, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for presence of aromatic rings
    ring_info = mol.GetRingInfo()
    aromatic_rings = []
    for ring in ring_info.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if any(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings.append(ring)
                
    if len(aromatic_rings) < 2:
        return False, "Requires at least 2 aromatic rings"
        
    # Check for presence of phenol or methoxy groups on aromatic rings
    has_phenol_or_methoxy = False
    for ring in aromatic_rings:
        ring_atoms = set(ring)
        for atom_idx in ring:
            atom = mol.GetAtomWithIdx(atom_idx)
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'O':
                    if neighbor.GetTotalNumHs() == 1 or any(n.GetSymbol() == 'C' and n.GetDegree() == 1 for n in neighbor.GetNeighbors()):
                        has_phenol_or_methoxy = True
                        break
                        
    if not has_phenol_or_methoxy:
        return False, "Requires at least one phenol or methoxy group on aromatic ring"
    
    # Check molecular weight range typical for lignans (adjusted range)
    mol_wt = Chem.Descriptors.ExactMolWt(mol)
    if mol_wt < 200 or mol_wt > 1000:  # Increased upper limit
        return False, f"Molecular weight {mol_wt:.1f} outside typical lignan range"
    
    # Count number of oxygen atoms (lignans typically have multiple oxygens)
    oxygen_count = len([atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'O'])
    if oxygen_count < 3:  # Reduced minimum oxygen requirement
        return False, "Insufficient oxygen atoms for typical lignan structure"
    
    # Count carbons to check for potential C6-C3 units
    carbon_count = len([atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'C'])
    if carbon_count < 15:  # Adjusted minimum for basic lignan skeleton
        return False, "Insufficient carbons for lignan skeleton"
    
    # Check for presence of aliphatic carbons connecting aromatic rings
    aliphatic_carbons = []
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and not atom.GetIsAromatic():
            aromatic_neighbors = sum(1 for n in atom.GetNeighbors() if n.GetIsAromatic())
            if aromatic_neighbors > 0:
                aliphatic_carbons.append(atom.GetIdx())
                
    if len(aliphatic_carbons) < 2:
        return False, "Missing characteristic lignan carbon framework"
        
    return True, "Contains required lignan structural features: phenylpropanoid units and characteristic oxygenation pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25036',
                          'name': 'lignan',
                          'definition': 'Any phenylpropanoid  derived from '
                                        'phenylalanine via dimerization of '
                                        'substituted cinnamic alcohols, known '
                                        'as monolignols, to a dibenzylbutane '
                                        'skeleton. Note that while individual '
                                        'members of the class have names '
                                        'ending ...lignane, ...lignene, '
                                        '...lignadiene, etc., the class names '
                                        'lignan, neolignan, etc., do not end '
                                        'with an "e".',
                          'parents': ['CHEBI:26004']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.2962962962962963 is too low.\n'
               'True positives: '
               "[('COc1cc(OC)c2cc3C(C)OC(C)Cc3c(-c3ccc4cc5CC(C)(O)CC(=O)c5c(O)c4c3O)c2c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C[C@H]([C@@](O)(CC2=CC(OC)=C(O)C=C2)C1=O)CC3=CC(OC)=C(O)C=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1ccc(cc1O)[C@H]1OC[C@H]2[C@@H]1CO[C@@H]2c1ccc(OC)c(O)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('[H][C@]12CO[C@H](c3ccc(O)c(OC)c3)[C@@]1([H])CO[C@@H]2c1ccc(O)c(OC)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('[H][C@]12COC(=O)[C@]1([H])Cc1cc(OC)c(O)cc1[C@@H]2c1ccc(O)c(OC)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), ('CC(Cc1ccc(O)c(O)c1)C(C)Cc1ccc(O)c(O)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(ccc1O)[C@@H]1OC[C@H](Cc2cc(OC)c(O)c(OC)c2)[C@H]1C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(ccc1O)[C@H]1OC[C@H]2[C@@H]1CO[C@@H]2c1cc(OC)c(O[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)c(OC)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O(CC1C(CC=2C(C1C3=CC(OC)=C(O)C(OC)=C3)=C(OC)C(O)=C(OC)C2)CO)C4OC(C(O)C(O)C4O)CO', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C2(OC(=O)C)C(C(OC2)C3=CC(OC)=C(O)C=C3)C1)C4=CC(OC)=C(O)C=C4', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(ccc1O)[C@H](O)[C@H]1CO[C@@H]([C@H]1CO)c1ccc(O)c(OC)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(cc(OC)c1O[C@@H](CO)[C@@H](O)c1ccc(O)cc1)-c1cc(=O)c2c(O)cc(O)cc2o1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COC=1C=C(C=C(C1O)OC)C(C2COC(C3=CC(=C(C(=C3)OC)OC(C(C=4C=CC(=C(C4)OC)O)O)CO)OC)C2COC(C5=CC=C(C=C5)O)=O)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(ccc1O)[C@H](O)[C@H](CO)Oc1c(OC)cc(O)cc1OC', 'Contains "
               'required lignan structural features: phenylpropanoid units, '
               "phenol groups, and characteristic oxygenation pattern'), "
               "('O1C(C(O)C2=CC3=C(C=4C=5C(C(O)=C6C4CC(C)OC6)=C(OC)C=C(C5)OC)C(OC)=CC(=C3C(=C2C1)O)OC)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(CC2=C(C=3C=4C(C(O)=C5C3CC(C)OC5)=C(OC)C=C(C4)OC)C6=CC(OC)=CC(=C6C(=C2C1)O)OC)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C[C@@H]([C@@H](CC2=C(O)C=CC(O)=C2)C1=O)CC3=CC(O)=CC=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C2C(C(OC2)C3=CC(OC)=C(O)C=C3)C1)C4=CC(OC)=C(OC5OC(C(O)C(O)C5O)CO)C=C4', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(cc(OC)c1O)[C@@H]1[C@@H](CO[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)[C@@H](CO)Cc2cc(OC)c(O)c(OC)c12', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(C[C@]2(O)[C@@H](Cc3ccc(OC)c(OC)c3)COC2=O)ccc1O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C1=C2OCOC2=CC(=C1)[C@]3(OC[C@]4([C@@]3(CO[C@@]4(C5=CC=C(C(=C5)O)O)[H])[H])[H])[H]', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(cc(OC)c1O)[C@H]1[C@@H](CO[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)[C@@H](CO)Cc2cc(OC)c(O)c(OC)c12', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C2(O)C(C(OC2)C3=CC(OC)=C(O)C=C3)C1)C4=CC(OC)=C(OC5OC(C(O)C(O)C5O)CO)C=C4', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cc(ccc1O)[C@@H]1O[C@@H]([C@@H](C)[C@@H]1C)c1ccc(OC)c(OC)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern')]\n"
               'False positives: '
               "[('CC(C)=CCc1ccc(O)c2c1oc1cc(C)c3OC[C@@H]([C@@H](O)c3c1c2=O)C(C)=C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C(O)=CC=2O[C@]3([C@H](O)C[C@@H]4[C@](OC=5C=C(O)C(C=C(C5C4)C)=O)(CC=CC(C[C@H]3CC2C(=C1)C)(C)C)C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(O)C=CC3=C2[C@@H](C=4C=5C(=C(O)C=CC5)C(=CC34)O)[C@H](C1)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COC(=O)c1c(O)cc2cc3C(=O)c4cc(OC)cc(O)c4C(=O)c3c(O)c2c1C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('OC1=CC=C2C3(OC(C4=C3C=C(C=C4)C(*)=O)=O)C=5C(OC2=C1)=CC(=CC5)O.OC6=CC=C7C8(OC(C9=C8C=CC(=C9)C(*)=O)=O)C=%10C(OC7=C6)=CC(=CC%10)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C2=C(OC=3C(C2=O)=C(O)C=C(O)C3)C4=C1C=C(O)C=C4)C=C(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(C1=C2OC(C=CC2=CC=C1O)(C)C)C3=C(O)C=C(C)C(=C3C=O)OC', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1NC=2C(=O)C3=C(O)C(=C(O)C(=C3C(C2)=O)C(=O)C(=CC4C(O)O[C@@H]([C@@H]([C@@H]([C@@H]([C@H]([C@H](C=CC=C1CO)C)O)C)O)C)[C@H](C)[C@H]4O)C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(O)C=3C(=CC(OC)=C(C3C)C(=O)OC)C=C2C(=O)[C@]4([C@]1(O)C(OC)=CC(=O)[C@@H]4OC)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('P(O[C@H](C(=O)N[C@H]([C@]1(OC(=O)C2=C(C1)C=CC=C2O)[H])CC(C)C)[C@@H](O)[C@@H](N)CC(=O)N)(O)(O)=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1[C@@H]([C@@H](O)CC2=C1C=C(OC)C=C2OC)C3=CC(OC)=C(O)C=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('OC(=O)CCc1cc2c(-c3cc(ccc3C(O)=O)C(O)=O)c3cc(CCC(O)=O)c(=O)cc3oc2cc1O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1[C@@H]([C@H](O)[C@H](O)[C@@H](O)C1OC2=C(OC=3C(C2=O)=C(O)C=C(O)C3)C4=CC(OC)=C(O)C=C4)CO[C@@]5(O[C@H]([C@H](O)[C@@H](O)[C@H]5O)C)[H]', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('CC(C)=CCc1c(oc2c(c(O)cc(O)c2c1=O)C(C)(C)C=C)-c1ccc(O)cc1O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(O)C3=C(O)C=C(C)C=C3[C@@H]([C@]24C=C[C@H]1C5=C6O[C@@](C(=O)OC)([C@H]7OC(=O)CC7)CC(C6=C(O)C=C5C4)=O)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(C=3C(=O)C=4C=CC=C(C4C3C(=C2)O)O)[C@@H](OC(=O)C(C)C)[C@@H]([C@H]1C)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(OC2=C(OC3=C(C2=O)C(OC)=CC(O)=C3)C4=CC(O)=C(O)C=C4)C(O)C(O)C(O)C1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('N(C=1C=2C(C(=CC1)/N=N/C=3C=4C(C=C(C3)S(=O)(=O)O)=CC(=CC4O)S(=O)(=O)O)=CC=CC2S(=O)(=O)O)C5=CC=CC=C5', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('[Na+].[Na+].[Na+].Oc1ccc(cc1C([O-])=O)C(=C1C=CC(=O)C(=C1)C([O-])=O)c1ccc(O)c(c1)C([O-])=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1[C@@H]([C@@H](O)[C@H](O)[C@@H](O)[C@@H]1OC2=C(O)C=C(C=C2)COC(=O)C3=CC(O)=C(O)C=C3)CO', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C1(C[C@@H](CC=2C1=C(C=3C(=C4[C@@]5(C=CO[C@@]5(OC4=CC3C2O)[H])[H])O)O)O)=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1cccc2C(=O)c3c(O)c4C[C@](O)(C[C@H](O[C@H]5C[C@H](N)[C@@H](O)[C@H](C)O5)c4c(O)c3C(=O)c12)C(=O)CO', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C(O)=C(O)C=2C=3O[C@@H](C)C(C3C(=C4C2C1=C(OC)C=C4C)O)(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C=CC=2C1=C(CC(O)C(C)=C)C(O)=C3C2OC=C(C3=O)C4=CC=C(O)C=C4)(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COc1c(C)c(O)c(Cc2c(O)c(C)c(OC)c(C(=O)C(C)C)c2O)c(O)c1C(=O)C(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('CC(C)=CCc1c(O)c(O)c(O)c2oc3cc4OC(Cc4c(O)c3c(=O)c12)C(C)=C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C=2C3=C(O)C=C(C)C=C3[C@@H](OC)[C@@H](C2C(=O)C4=C1C=CC(=C4O)[C@@H]5O[C@@H]([C@@H](O)[C@@H](C5)O[C@@H]6O[C@H]([C@@H](O[C@@H]7O[C@@H]([C@@H](O)[C@@H](C7)O)C)CC6)C)C)OC', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(=O)C([C@H](CC)C2=CC=CC=C2)=C(O)C3=C1C=CC(O)=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('S(OC=1C=C(C=2OC3=C(C(O)=C(C4OC(C(=O)C(O)C4OC5OC(C(O)C(O)C5O)C)C)C(O)=C3)C(=O)C2)C=CC1O)(O)(=O)=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C2=C(C(=CC(=C2)OC3=CC(O)=CC(=C3)C)C)[C@H]4OCC=5C4=C1C=CC5', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(CC2=CC3=C(OC(C=C3)(C)C)C=C2)(C(=C(OC)C1=O)C4=CC(O)=C(O)C=C4)C(OC)=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1[C@@H](O)C2=C(O)C=C3C=4C=CC(=C5C4C(C3=C2[C@@H]6[C@H]1O6)=CC=C5O)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C2C(C=3C1=CC(=CC3O)CCCCC)C=C(CC2)C(O)=O)(C)C', 'Contains "
               'required lignan structural features: phenylpropanoid units, '
               "phenol groups, and characteristic oxygenation pattern'), "
               "('O=C1C2=C(C(O)=CC=C2CC(C1)(O)C)[C@@H]3O[C@@H](OC)C4=C3C=CC=C4OC', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(C1=C(C(O)=CC=C1CC=C(C)C)C=O)C2=C(O)C(OC[C@H](O)C(O)(C)C)=CC(=C2)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1O[C@](C(=O)OCC)(CC2=CC3=C(OC(C)(C)C(C3)O)C=C2)C(=C1O)C4=CC=C(O)C=C4', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(O)C(=CC=C2C(=O)C3=C1C=C(OC)C4=C3C(=O)C[C@@](O)(C)C4)[C@@H]5O[C@@H]([C@H]6O[C@@H]7O[C@@H](C)C(C[C@@H]7O[C@@H]6C5)=O)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('OC[C@@H]1O[C@@H](O[C@@H]2[C@H](Oc3cc(O)cc(O)c3C2=O)c2ccc(O)c(O)c2)[C@H](O)[C@H]1O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('OC[C@H]1O[C@H]([C@H](O)[C@@H](O)[C@@H]1O)c1c(O)cc(O)c2c1oc1cc(O)c(O)cc1c2=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1OC(C2=C(O)C(=CC3=C2C(=O)C=C(N)C3=O)C)=C(C)C=C1C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(OC=2C(=C3OC(C=CC3=C4OC(=O)C=C(C24)C5=CC=C(O)C=C5)(C)C)C(=O)C(C)C)C(O)C(O)C(O)C1C(O)=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C(O)(C)C)CC2=C1C=C3OC(=O)C(=CC3=C2OC)C4=C(O)C=C(O)C=C4', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('Oc1ccc(cc1)[C@@H]1Oc2cc(O)cc(O)c2C(=O)[C@H]1c1c(O)cc(O)c2c1oc(cc2=O)-c1ccc(O)cc1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('S(C1=C(NC)C(=O)C=2C=C3C[C@H](CC(=O)O)O[C@@H](C3=C(C2C1=O)O)CCC(C)C)CC(NC(=O)C)C(=O)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C1[C@]2([C@]3([C@@](C4=C(C=C(O)C=C4)CC3)(CC[C@@]2([C@@H](O)[C@@H]1O[C@@H]5O[C@@H]([C@H]([C@@H]([C@H]5O)O)O)C(O)=O)C)[H])[H])[H]', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('OC1(C2=C(C(=O)C3=C1C=C(OC)C=C3O)C(O)=CC(=C2)C)CC=C(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('ClC1=C(OC)C=C(O)C2=C1[C@](O[C@@H]3O[C@H]([C@H](OC)[C@](C3)(NO)C)C)([C@H]4C[C@@]5(O)[C@H](N(C)C)C(=O)C(=C([C@]5(C(C4=C2O)=O)O)O)C(=O)N)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C3C(O[C@@]45C=6C(=C(O)C=CC6O[C@@]3(O4)[C@@H](C1)O)C(=O)CC5)=CC=C2', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(OC3=C1C4=C(OC[C@@H]([C@H]4O)C(=C)C)C(=C3)C)C(=CC=C2O)CC(=O)C(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1N(O)C=C(C2=CC=C(O[C@H]3OC([C@H](OC)[C@@H](C3O)O)CO)C=C2)C(=C1C4OC(CCC4C)C)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C[C@@H]1O[C@@H](Oc2cc(O)c3c(c2)oc(-c2ccc(O)cc2)c(O[C@@H]2O[C@@H](C)[C@H](O)[C@@H](O)[C@H]2O)c3=O)[C@H](O)[C@H](O)[C@H]1O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(CC(=O)C=2C(O)=C3C(CC(OC3=CC12)=O)C4=CC=CC=C4)C5=CC=C(O)C=C5', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1O[C@H](C2=C(O)C=CC(=C2)O)C=C1CC/C=C(\\\\CO)/CC/C=C(/COC(=O)/C=C\\\\C3=CC=C(O)C=C3)\\\\C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C2=C(C(O)=C(CC=C(C)C)C(O)=C2)C(=O)C=C1C3=CC(CC=C(C)C)=C(O)C=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('N1(CCC2=C(C(CC3=C(C1)C(=C(C=C3)O)OC)=O)C=C4C(=C2)OCO4)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(C(O)=C(C3=C(O)C=4C(=O)[C@H]5[C@@H](C[C@H](C)O[C@@H]5C)[C@H](C4C=C3OC)O)C(=C2)OC)C(=O)[C@H]6[C@@]1(O)C[C@H](C)O[C@@H]6C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(OC)C1=C(O)C2=C(OC(C)=C[C@@]2([C@@H]3[C@@]4(OC=5C=C(C)C(=C(C5[C@H]3C=6C(O)=CC(=CC6O4)C)O)C(=O)OC)C)C)C=C1C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C1(C(=C([C@H]([C@@]2(C[C@@]3(CC4=C(C=C(C(=C4C(C3=C([C@]12O)O)=O)O)NC(C[NH2+]C(C)(C)C)=O)N(C)C)[H])[H])[NH+](C)C)[O-])C(=O)N)=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(O[C@@H](C1)C[C@@H](O)C)C=C(O)C=C2CC(=O)OCC(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('CC(C)=CCc1c(O)c(CC=C(C)C)c2O[C@@H](CC(=O)c2c1O)c1cc(O)cc(O)c1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\Cc1c(O)c2C[C@H](O)C(C)(C)Oc2c2c1oc1ccc(O)cc1c2=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C(OC)=C(OC2=C1C(O)=C(OC)C(=C2C)OC)C3=CC=C(O)C=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C2OC3=C(C2O)C=4N(C=5C(C(=O)C4C(O)=C3)=CC=CC5)C)(C1)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(C1=C2OC(C=CC2=CC=C1OC)(C)C)C3=C(O)C=C(C)C4=C3[C@@H](O)[C@@H](C(=C)C)CO4', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C1(C=C(CC=2C1=C(C=3C(=C4[C@@]5(C=CO[C@@]5(OC4=CC3C2O)[H])[H])O)O)O)=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(OC3=C1C(=CC(=C3)O)C)C=C(O)C=C2CC(=O)C[C@H](O)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(OCC1=CC(O)=C([C@@](O)(CCCC(C)C)C)C=C1)C2=CC(O)=C([C@@](O)(CCCC(C)C)C)C=C2', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(C1C(CC(=CC1C=2C(O)=C(OC)C(=CC2O)/C=C/C3=C(O)C=C(O)C=C3)C)C4=C(O)C=C(O)C=C4)C5=C(O)C(=C(O)C=C5)CC=C(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1NCC2=C1C=C(O)C3=C2OC(C=C3)(CC/C=C(\\\\CCC(O)C(O)(C)C)/C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(OC1=C(C(O)=C(O)C2=C1OC=3C=C(O)C(=CC23)O)C4=CC=C(O)C=C4)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('[H]C1C[C@H](C)OC(=O)C[C@@H](NC(=O)[C@@H](Cc2c(Br)[nH]c3ccccc23)N(C)C(=O)[C@H](C)NC(=O)[C@@H](C)C\\\\C(C)=C\\\\1)c1ccc(O)cc1', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C2=C(C(=O)C(OC)=C1C3=CC=4OCOC4C=C3)C(O)=CC(OCC=C(C)C)=C2OC', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(C(=O)C=3C1=C(O)C=CC3)C4=C(C(=O)[C@@](O)(C(C)C)CC4=O)C=C2O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C(OC1=C(C(O)=C(C(=O)OC)C(=C1C)C)C)C2=C(O)C(=C(O)C(=C2C)C)C=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C=2C(C(O)=C(/C(/C3=CC=CC=C3)=C/C(=O)C)C1=O)=CC=CC2', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C(O)=C(C(=O)C(=C1C2=CC=C(OC)C=C2)O)C3=CC=C(O)C=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O(C=1C(C2=C(O)C3=C(C(=O)C2=O)C(O)=CC=C3)=C(C=C(C1)C)C(O)=O)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=CN/C=C/C1=CC(=C(O)C=C1)C2=C(O)C=CC(=C2)/C=C/NC=O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1OC2=C(C(=CC(=C2C)O)C)OC3=C1C(=CC(=C3CC4=C(OC)C=C(O)C=C4C)O)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C1(=C(CC=C(C)C)C(=C(C(=O)/C=C/C2=CC=C(C=C2)O)C(=C1)O)O)OC', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(C(=O)C=3C1=C(O)C4=C([C@@H](C(=O)OC)[C@@](O)(CC)C[C@@H]4OC5OC(C(OC6OC(C7OC8OC(C)C(CC8OC7C6)=O)C)CC5)C)C3)C(O)=CC=C2O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1N[C@H](C(=O)N[C@H](C(=O)NC(C(=O)N[C@H](C(=O)O)CO)CCCN=C(N)N)[C@H](O)C2=C(O)C=CC(C3=CC(C[C@@H]1N)=C(O)C=C3)=C2)C[C@@H](O)CN', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C(O)CC2=C1C=C(O)C=3OC4=C(C(=O)C23)C(O)=C(C(O)=C4)CC=C(C)C)(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(OC2=C(O)C(=[N+]([O-])C=C2)C3=[N+]([O-])C=CC(=C3O)OC4OC(C(O)C(C4O)O)CO)C(O)C(O)C(C1CO)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(OC3=C1C4=C(OC[C@H]([C@H]4O)C(=C)C)C(=C3)C)C(=CC=C2O)[C@@H](O)[C@@H](O)C(O)(C)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1OC(=CC(=C1C2=C(C=C(O)C(=C2)O)C=CC=3OC(=O)C=C(C3)O)O)C=CC4=CC(O)=C(O)C=C4', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('C1OC(C2COC(C12)C3=CC(=C(C(=C3)OC)O)OC)C4=CC=C(C(=C4)OC)OC(C(C=5C=C(C(=CC5)O)OC)O)CO', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(C(=O)C=3C1=C(O)C=CC3)C=4C(=O)C[C@@H](C)CC4C=C2O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C=2C(OC)=CC(O)=C(O)C2)=C(OC)C(=O)C3=C1C=C(O)C(OC)=C3O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1OC(=CC=2C1=C(O)C(=C(O)C2)C(=O)C3=C(O)C=C(OC)C=C3C(=O)OC)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), ('O=C(OC1=CC(O)=CC(=C1)CCC)C2=C(O)C=C(OC)C=C2CCC', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('[H][C@@]12Cc3ccc(OC)c(OC)c3CN1CCc1cc(OC)c(O)cc21', 'Contains "
               'required lignan structural features: phenylpropanoid units, '
               "phenol groups, and characteristic oxygenation pattern'), "
               "('O1[C@@H](OC2=C(O)C(=C(O)C=C2C3=CC=C(OC)C=C3)C4=CC=C(OC)C=C4)[C@H](O)[C@@H](O)[C@@H]([C@H]1CO)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1C2=C(C(=O)C=3C1=C(O)C4=C([C@@H](O[C@@H]5O[C@@H]([C@H](O)[C@H](C5)N(C)C)C)C[C@]([C@@H]4O)(O)CC)C3)C(O)=CC=C2O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O=C1NC=C(C2=CC=C(O)C=C2)C(=C1C(=O)[C@@H]3C=C([C@H]4CC[C@@H](C[C@@H]4[C@H]3/C=C/CO)C)C)O', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('COC1=CC(=CC(=C1O)OC)C=C2C(=O)NC(=O)N(C2=O)C3=CC=CC=C3', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('ClC1=C2OC=3N(C(Cl)=CC3C(C2=C(O)C(=C1)C)=O)[C@@H]4C=C([C@@H](OC)[C@@H]([C@H]4O)O)CO', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O(C=1C=C(C(C(C)C)(CCCN(CCC2=CC(OC)=C(OC)C=C2)C)C#N)C=CC1O)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1[C@@H]([C@@H](O)[C@H](O)[C@@H](O[C@@H]2OC[C@@H](O)[C@H](O)[C@H]2O)[C@@H]1OC3=C(OC=4C(C3=O)=C(O)C=C(O)C4)C5=CC=C(O)C=C5)CO[C@@H]6O[C@H]([C@H](O)[C@@H](O)[C@H]6O)C', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern'), "
               "('O1C(C(O)C(C=2C=3OC(C(O)CC3C(O)=CC2O)C4=CC(O)=C(O)C=C4)C=5C1=CC(O)=CC5O)C6=CC=C(O)C=C6', "
               "'Contains required lignan structural features: phenylpropanoid "
               'units, phenol groups, and characteristic oxygenation '
               "pattern')]\n"
               'False negatives: '
               "[('O1[C@@H](OC2=C(OC)C=C([C@H]3OC[C@@H]4[C@@H]3CO[C@H]4C5=CC(OC)=C(O[C@@H]6O[C@@H]([C@@H](O)[C@@H]([C@H]6O)O)CO)C(=C5)OC)C=C2OC)[C@H](O)[C@@H](O)[C@@H]([C@H]1CO)O', "
               "'Requires at least one phenol group'), "
               "('C1OC(C2COC(C12)C3=CC(=C(C(=C3)OC)O)OC)C4=CC(=C(C(=C4)OC)OC(C(C=5C=C(C(=C(C5)OC)OC(C(C=6C=C(C(=CC6)O)OC)O)CO)OC)O)CO)OC', "
               "'Molecular weight 840.3 outside typical lignan range'), "
               "('COc1cc(cc(OC)c1OC(C)=O)C1OCC2C1COC2c1cc(OC)c(OC(C)=O)c(OC)c1', "
               "'Requires at least one phenol group'), "
               "('O(C=1C=C(C[C@@H]([C@@H](CC2=CC(OC)=C(OC)C=C2)C)C)C=CC1OC)C', "
               "'Requires at least one phenol group'), "
               "('COc1cc(cc2OCOc12)[C@@H]1O[C@H]([C@H](C)[C@H]1C)c1cc(OC)c2OCOc2c1', "
               "'Requires at least one phenol group'), "
               "('O(CC(C(COC1OC(C(O)C(O)C1O)CO)CC2=CC(OC)=C(O)C=C2)CC3=CC(OC)=C(O)C=C3)C4OC(C(O)C(O)C4O)COC(=O)CC(O)(CC(O)=O)C', "
               "'Molecular weight 830.3 outside typical lignan range'), "
               "('O1CC2=C(C=3C(C(OC)=C2C1=O)=CC=4OCOC4C3)C5=CC=6OCOC6C=C5', "
               "'Requires at least one phenol group'), "
               "('COc1ccc(cc1OC)[C@H]1O[C@H]([C@H](C)[C@@H]1C)c1ccc2OCOc2c1', "
               "'Requires at least one phenol group'), "
               "('COC[C@H]1O[C@@H](O[C@@H]2CO[C@@H](O[C@@H]3[C@@H](CO)O[C@@H](Oc4c5COC(=O)c5c(-c5ccc6OCOc6c5)c5cc(OC)c(OC)cc45)[C@H](O)[C@H]3O)[C@H](OC)[C@H]2OC)[C@H](OC)[C@H]1OC', "
               "'Requires at least one phenol group'), "
               "('Oc1ccc(CC(=C)C(=C)Cc2ccc(O)cc2)cc1', 'Insufficient oxygen "
               "atoms for typical lignan structure'), "
               "('O[C@@H]([C@H]1COC(=O)[C@@H]1C(=O)c1ccc2OCOc2c1)c1ccc2OCOc2c1', "
               "'Requires at least one phenol group'), "
               "('C[C@@H]1[C@@H](C)[C@@H](O[C@H]1c1ccc2OCOc2c1)c1ccc2OCOc2c1', "
               "'Requires at least one phenol group'), "
               "('COc1ccc(C[C@H]2COC(=O)[C@]2(O)Cc2ccc(O[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)c(OC)c2)cc1OC', "
               "'Requires at least one phenol group'), "
               "('COc1cc(cc(OC)c1OC)[C@H]1[C@@H]2[C@H](COC2=O)Cc2c1cc1OCOc1c2OC', "
               "'Requires at least one phenol group')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 33,
    'num_false_positives': 100,
    'num_true_negatives': 998,
    'num_false_negatives': 5,
    'num_negatives': None,
    'precision': 0.24812030075187969,
    'recall': 0.868421052631579,
    'f1': 0.38596491228070173,
    'accuracy': 0.9075704225352113}


"""
Classifies: CHEBI:140997 omega-hydroxy-long-chain fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_omega_hydroxy_long_chain_fatty_acid(smiles: str):
    """
    Determines if a molecule is an omega-hydroxy-long-chain fatty acid (C13-C22).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an omega-hydroxy-long-chain fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)O')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"
        
    # Check for terminal hydroxyl group (omega position)
    terminal_oh_pattern = Chem.MolFromSmarts('CCO')
    if not mol.HasSubstructMatch(terminal_oh_pattern):
        return False, "No terminal hydroxyl group found"
        
    # Count total carbons
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    
    if carbon_count < 13:
        return False, f"Chain too short ({carbon_count} carbons)"
    if carbon_count > 22:
        return False, f"Chain too long ({carbon_count} carbons)"
    
    # Count hydroxyl groups (excluding the carboxylic acid OH)
    oh_pattern = Chem.MolFromSmarts('O[CH]')  # Matches hydroxyl groups attached to carbon
    oh_count = len(mol.GetSubstructMatches(oh_pattern))
    
    # At least one hydroxyl group should be terminal (omega position)
    if oh_count < 1:
        return False, "No hydroxyl groups found"
    
    return True, f"Omega-hydroxy fatty acid with {carbon_count} carbons and {oh_count} hydroxyl groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140997',
                          'name': 'omega-hydroxy-long-chain fatty acid',
                          'definition': 'A omega-hydroxy-fatty acid with a '
                                        'chain length ranging from C13 to C22.',
                          'parents': ['CHEBI:10615', 'CHEBI:15904']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Invariant Violation\n'
               '\t\n'
               '\tViolation occurred on line 341 in file '
               'Code/GraphMol/Matrices.cpp\n'
               '\tFailed Expression: aid1 != aid2\n'
               '\tRDKIT: 2024.03.6\n'
               '\tBOOST: 1_85\n',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 1588,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9409332545776727}


"""
Classifies: CHEBI:24960 ketoaldehyde
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_ketoaldehyde(smiles: str):
    """
    Determines if a molecule is a ketoaldehyde (i.e., contains both an aldehydic and ketonic group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a ketoaldehyde, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all aldehyde and ketone groups
    aldehydes = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'C' and atom.GetHybridization() == Chem.HybridizationType.SP2 and len(atom.GetNeighbors()) == 2 and any(neighbor.GetSymbol() == 'O' and neighbor.GetIsAromatic() is False for neighbor in atom.GetNeighbors())]
    ketones = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'C' and atom.GetHybridization() == Chem.HybridizationType.SP2 and len(atom.GetNeighbors()) == 3 and any(neighbor.GetSymbol() == 'O' and neighbor.GetIsAromatic() is False for neighbor in atom.GetNeighbors())]

    if not aldehydes or not ketones:
        return False, "No aldehyde or ketone groups found"

    return True, "Molecule contains both an aldehyde and ketone group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24960',
                          'name': 'ketoaldehyde',
                          'definition': 'Any compound that has an aldehydic '
                                        'and ketonic group in the same '
                                        'molecule.',
                          'parents': ['CHEBI:17087', 'CHEBI:17478']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 4647,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.97893850042123}


"""
Classifies: CHEBI:28868 fatty acid anion
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_fatty_acid_anion(smiles: str):
    """
    Determines if a molecule is a fatty acid anion.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a fatty acid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for carboxylate group
    has_carboxylate = any(atom.GetFormalCharge() == -1 and atom.GetTotalNumHs() == 0
                          and sum(bond.GetBondTypeAsDouble() for bond in atom.GetBonds()) == 3
                          for atom in mol.GetAtoms())
    if not has_carboxylate:
        return False, "No carboxylate group found"

    # Check for carbon chain
    carbon_chain_length = rdMolDescriptors.CalcTPSA(mol) / 9.23
    if carbon_chain_length < 4:
        return False, "Carbon chain too short"

    return True, f"Fatty acid anion with carbon chain length {int(carbon_chain_length)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28868',
                          'name': 'fatty acid anion',
                          'definition': 'The conjugate base of a fatty acid, '
                                        'arising from deprotonation of the '
                                        'carboxylic acid group of the '
                                        'corresponding fatty acid.',
                          'parents': ['CHEBI:18059', 'CHEBI:35757']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.Descriptors' has no "
               "attribute 'WHDPNBWMLSJ1024'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 100703,
    'num_false_negatives': 99,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 0.01,
    'f1': 0.009950248756218905,
    'accuracy': 0.9980278088857616}


"""
Classifies: CHEBI:26244 prenols
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_prenols(smiles: str):
    """
    Determines if a molecule is a prenol (alcohol with isoprene units).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a prenol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for presence of OH group(s)
    oh_pattern = Chem.MolFromSmarts("[OH]")
    if not mol.HasSubstructMatch(oh_pattern):
        return False, "No hydroxyl group found"
        
    # Pattern for isoprene unit: -CH2-C(CH3)=CH-CH2-
    isoprene_pattern = Chem.MolFromSmarts("[CH2][C](C)=[CH][CH2]")
    if not mol.HasSubstructMatch(isoprene_pattern):
        return False, "No isoprene unit found"
        
    # Count carbons and double bonds
    num_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    num_double_bonds = rdMolDescriptors.CalcNumAliphaticDoubleBonds(mol)
    
    # Check if carbon count follows isoprene rule (5n where n is number of units)
    if (num_carbons - 1) % 5 != 0:  # -1 because of terminal CH2OH
        return False, "Carbon count does not match isoprene rule"
        
    # Number of isoprene units
    n_units = (num_carbons - 1) // 5
    
    # Check number of double bonds (should be n for linear prenols)
    if num_double_bonds != n_units:
        return False, "Number of double bonds does not match expected for linear prenol"
        
    # Additional check for terminal CH2OH group
    terminal_oh_pattern = Chem.MolFromSmarts("[CH2]OH")
    if not mol.HasSubstructMatch(terminal_oh_pattern):
        return False, "No terminal CH2OH group found"
    
    return True, f"Prenol with {n_units} isoprene unit(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26244',
                          'name': 'prenols',
                          'definition': 'Any alcohol possessing the general '
                                        'formula H-[CH2C(Me)=CHCH2]nOH in '
                                        'which the carbon skeleton is composed '
                                        'of one or more isoprene units '
                                        '(biogenetic precursors of the '
                                        'isoprenoids).',
                          'parents': ['CHEBI:24913', 'CHEBI:30879']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.rdMolDescriptors' has no attribute "
             "'CalcNumAliphaticDoubleBonds'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:176842 vitamin B9
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_vitamin_B9(smiles: str):
    """
    Determines if a molecule is vitamin B9 (folic acid) or a derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is vitamin B9, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for key structural features of vitamin B9
    
    # 1. Must contain pterin core (2-amino-4-oxo-3,4-dihydropteridine)
    # Multiple SMARTS patterns to catch different representations
    pterin_patterns = [
        Chem.MolFromSmarts('c1nc2c(n1)nc([nH]c2=O)N'),  # Basic pattern
        Chem.MolFromSmarts('c1nc2c(nc(N)nc2[nH]1)=O'),  # Alternative pattern
        Chem.MolFromSmarts('N=c1[nH]c(=O)c2nc[nH]c2[nH]1'),  # Another form
        Chem.MolFromSmarts('Nc1nc2NCC3CN(CN3c2c(=O)[nH]1)'),  # THF form
        Chem.MolFromSmarts('Nc1nc2N=CN(CN2c(=O)[nH]1)'),  # Another variant
    ]
    
    has_pterin = any(mol.HasSubstructMatch(pattern) for pattern in pterin_patterns)
    if not has_pterin:
        return False, "Missing pterin core structure"

    # 2. Must contain p-aminobenzoic acid (PABA) moiety with connecting chain
    paba_patterns = [
        Chem.MolFromSmarts('c1ccc(C(=O)NC)cc1'),
        Chem.MolFromSmarts('c1ccc(cc1)C(=O)N[CH]'),
    ]
    
    has_paba = any(mol.HasSubstructMatch(pattern) for pattern in paba_patterns)
    if not has_paba:
        return False, "Missing p-aminobenzoic acid moiety"

    # 3. Must contain glutamic acid moiety (including ionized forms)
    glutamate_patterns = [
        Chem.MolFromSmarts('NC(CCC(=O)O)C(=O)O'),
        Chem.MolFromSmarts('NC(CCC([O-])=O)C([O-])=O'),
        Chem.MolFromSmarts('NCC(CCC(=O)*)C(=O)*'),  # More general pattern
    ]
    
    has_glutamate = any(mol.HasSubstructMatch(pattern) for pattern in glutamate_patterns)
    if not has_glutamate:
        return False, "Missing glutamic acid moiety"

    # If all required structural features are present
    return True, "Contains pterin core, PABA, and glutamate moieties characteristic of vitamin B9"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:176842',
                          'name': 'vitamin B9',
                          'definition': 'Any B-vitamin that exhibits '
                                        'biological activity against vitamin '
                                        'B9 deficiency. Vitamin B9 refers to '
                                        'the many forms of folic acid and its '
                                        'derivatives, including '
                                        'tetrahydrofolic acid (the active '
                                        'form), methyltetrahydrofolate (the '
                                        'primary form found in blood), '
                                        'methenyltetrahydrofolate, folinic '
                                        'acid amongst others. They are present '
                                        'in abundance in green leafy '
                                        'vegetables, citrus fruits, and animal '
                                        'products. Lack of vitamin B9 leads to '
                                        'anemia, a condition in which the body '
                                        'cannot produce sufficient number of '
                                        'red blood cells. Symptoms of vitamin '
                                        'B9 deficiency include fatigue, muscle '
                                        'weakness, and pale skin.',
                          'parents': ['CHEBI:75769']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('[H][C@]12CNc3nc(N)[nH]c(=O)c3[N+]1=CN(C2)c1ccc(cc1)C(=O)N[C@@H](CCC([O-])=O)C([O-])=O', "
               "'Missing pterin core structure'), "
               "('CN(CC1=CN=C2C(=N1)C(=NC(=N2)N)N)C3=CC=C(C=C3)C(=O)NC(CCC(=O)O)C(=O)O', "
               "'Missing pterin core structure'), "
               "('Nc1nc2NCC3CN(CN3c2c(=O)[nH]1)c1ccc(cc1)C(=O)N[C@@H](CCC([O-])=O)C([O-])=O', "
               "'Missing pterin core structure')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 1,
    'num_true_negatives': 183896,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.5,
    'recall': 0.3333333333333333,
    'f1': 0.4,
    'accuracy': 0.9999836867862969}


"""
Classifies: CHEBI:27208 unsaturated fatty acid
"""
from rdkit import Chem
from rdkit.Chem import MolFromSmiles, Descriptors

def is_unsaturated_fatty_acid(smiles: str):
    """
    Determines if a molecule is an unsaturated fatty acid (contains at least one C=C or C#C bond).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is an unsaturated fatty acid, False otherwise
        str: Reason for classification
    """
    mol = MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if molecule contains a carboxylic acid group
    if not any(atom.GetSymbol() == 'O' and atom.GetDegree() == 1 for atom in mol.GetAtoms()):
        return False, "No carboxylic acid group found"

    # Check if molecule contains at least one C=C or C#C bond
    unsaturated = False
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.DOUBLE or bond.GetBondType() == Chem.BondType.TRIPLE:
            begin_atom = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
            end_atom = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
            if begin_atom.GetSymbol() == 'C' and end_atom.GetSymbol() == 'C':
                unsaturated = True
                break

    if unsaturated:
        return True, "Molecule contains at least one C=C or C#C bond"
    else:
        return False, "No C=C or C#C bonds found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27208',
                          'name': 'unsaturated fatty acid',
                          'definition': 'Any fatty acid containing at least '
                                        'one C=C or C#C bond.',
                          'parents': ['CHEBI:35366']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 105,
    'num_false_positives': 100,
    'num_true_negatives': 230,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.5121951219512195,
    'recall': 0.9813084112149533,
    'f1': 0.6730769230769231,
    'accuracy': 0.7665903890160183}


"""
Classifies: CHEBI:25171 mannosyl groups
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_mannosyl_groups(smiles: str):
    """
    Determines if a molecule contains mannosyl groups.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains mannosyl groups, False otherwise
        str: Reason for classification
    """
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for pyranose ring pattern characteristic of mannose
    mannose_pattern = Chem.MolFromSmarts('[C@@H]1[C@H]([C@H]([C@@H]([C@H](O1)CO)O)O)O')
    if not mol.HasSubstructMatch(mannose_pattern):
        return False, "No mannose pyranose ring pattern found"
        
    # Check for glycosidic linkage (removal of hemiacetal OH)
    if '*' not in smiles:
        return False, "No glycosidic linkage point found"
        
    # Count number of mannose units
    matches = mol.GetSubstructMatches(mannose_pattern)
    num_mannose = len(matches)
    
    # Additional checks for correct stereochemistry at C2 position
    c2_pattern = Chem.MolFromSmarts('[C@@H]([C@H]1O[C@H]([C@H]([C@@H]([C@H]1O)O)O)CO)O')
    if not mol.HasSubstructMatch(c2_pattern):
        return False, "Incorrect stereochemistry for mannose"
        
    # Check if terminal mannose has glycosidic linkage
    terminal_pattern = Chem.MolFromSmarts('[C@@H]1[C@H]([C@H]([C@@H]([C@H](O1)CO)O)O)O*')
    if not mol.HasSubstructMatch(terminal_pattern):
        return False, "Terminal mannose unit not properly linked"
        
    return True, f"Contains {num_mannose} mannosyl group(s) with correct stereochemistry and glycosidic linkage"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25171',
                          'name': 'mannosyl groups',
                          'definition': 'A glycosyl group obtained by removing '
                                        'the hydroxy group from the hemiacetal '
                                        'function of a mannose and, by '
                                        'extension, of a lower oligosaccharide '
                                        'having mannose at the reducing end.',
                          'parents': ['CHEBI:24403']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183910,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999891252338075}


"""
Classifies: CHEBI:145555 macropolylide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_macropolylide(smiles: str):
    """
    Determines if a molecule is a macropolylide (macrocyclic ring with multiple ester linkages).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a macropolylide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Get ring information
    rings = mol.GetRingInfo()
    if not rings.NumRings():
        return False, "No rings found"

    # Find macrocyclic rings (size >= 12)
    macro_rings = [ring for ring in rings.AtomRings() if len(ring) >= 12]
    if not macro_rings:
        return False, "No macrocyclic rings (size >= 12) found"

    # Check for ester linkages in macrocyclic rings
    ester_count = 0
    for ring in macro_rings:
        ring_atoms = set(ring)
        
        # Find ester groups (-O-C(=O)-)
        for atom_idx in ring:
            atom = mol.GetAtomWithIdx(atom_idx)
            if atom.GetSymbol() == 'O':
                neighbors = [n for n in atom.GetNeighbors()]
                if len(neighbors) == 2:
                    for n in neighbors:
                        if n.GetSymbol() == 'C':
                            for nn in n.GetNeighbors():
                                if nn.GetSymbol() == 'O' and nn.GetIsAromatic() == False and nn.GetDegree() == 1:
                                    # Found ester group where both O and C are in the ring
                                    if n.GetIdx() in ring_atoms:
                                        ester_count += 1

    if ester_count == 0:
        return False, "No ester linkages found in macrocyclic ring"
    elif ester_count == 1:
        return False, "Only one ester linkage found (macrolide but not macropolylide)"
    
    # Classify based on number of ester linkages
    if ester_count == 2:
        return True, "Macrodiolide (2 ester linkages)"
    elif ester_count == 3:
        return True, "Macrotriolide (3 ester linkages)" 
    elif ester_count == 4:
        return True, "Macrotetrolide (4 ester linkages)"
    elif ester_count == 5:
        return True, "Macropentolide (5 ester linkages)"
    else:
        return True, f"Macropolylide with {ester_count} ester linkages"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:145555',
                          'name': 'macropolylide',
                          'definition': 'Macrolides (macrocyclic lactones) in '
                                        'which the macrocyclic ring contains '
                                        'more than one ester linkage. '
                                        'Macropolylides include macrodiolides, '
                                        'macrotriolides, macrotetrolides and '
                                        'macropentolides, each containing di-, '
                                        'tri-, tetra-, and penta- ester '
                                        'linkages, respectively, in one '
                                        'macrocyclic ring. Macrocyclic '
                                        'lactones containing nitrogen in their '
                                        'skeletons (azamacrolides and '
                                        'macrolide lactams) and also '
                                        'containing oxazole or thiazole in '
                                        'their skeletons are known in nature.',
                          'parents': ['CHEBI:25106']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 19221,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9948248201625006}


"""
Classifies: CHEBI:24848 inositol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_inositol(smiles: str):
    """
    Determines if a molecule is an inositol (any cyclohexane-1,2,3,4,5,6-hexol).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an inositol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count the number of rings
    ring_info = mol.GetRingInfo()
    rings = ring_info.AtomRings()
    if len(rings) != 1:
        return False, "Molecule does not contain exactly one ring"

    # Check if the ring is a 6-membered ring
    ring = rings[0]
    if len(ring) != 6:
        return False, "Molecule does not contain a 6-membered ring"

    # Check if all ring atoms are carbon
    ring_atoms = [mol.GetAtomWithIdx(idx) for idx in ring]
    if not all(atom.GetSymbol() == 'C' for atom in ring_atoms):
        return False, "Ring contains non-carbon atoms"

    # Check if all ring atoms are bound to a hydroxy group
    for atom in ring_atoms:
        hydroxy_groups = sum(1 for neighbor in atom.GetNeighbors() if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 1)
        if hydroxy_groups != 1:
            return False, "Ring atom not bound to a single hydroxy group"

    return True, "Molecule is an inositol (cyclohexane-1,2,3,4,5,6-hexol)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24848',
                          'name': 'inositol',
                          'definition': 'Any cyclohexane-1,2,3,4,5,6-hexol.',
                          'parents': ['CHEBI:23451', 'CHEBI:37206']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.GetAtomWithIdx(Mol, Atom)\n'
               'did not match C++ signature:\n'
               '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int '
               'idx)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 2,
    'num_true_negatives': 183923,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.3333333333333333,
    'recall': 1.0,
    'f1': 0.5,
    'accuracy': 0.9999891260615682}


"""
Classifies: CHEBI:23446 cyclic monoterpene ketone
"""
from rdkit import Chem
from rdkit.Chem import Descriptors

def is_cyclic_monoterpene_ketone(smiles: str):
    """
    Determines if a molecule is a cyclic monoterpene ketone.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a cyclic monoterpene ketone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule is cyclic
    if not Chem.GetSSSR(mol):
        return False, "Not a cyclic molecule"

    # Check if the molecule contains a carbonyl group
    has_carbonyl = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and sum(bond.GetBondType() == Chem.BondType.DOUBLE for bond in atom.GetBonds()) == 1:
            has_carbonyl = True
            break
    if not has_carbonyl:
        return False, "No carbonyl group found"

    # Check if the molecule is a monoterpene
    num_carbon_atoms = sum(atom.GetSymbol() == 'C' for atom in mol.GetAtoms())
    if num_carbon_atoms != 10:
        return False, "Not a monoterpene (incorrect number of carbon atoms)"

    # If all conditions are met, it's a cyclic monoterpene ketone
    return True, "Cyclic monoterpene ketone"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23446',
                          'name': 'cyclic monoterpene ketone',
                          'definition': 'A cyclic terpene ketone in which the '
                                        'terpene specified is monoterpene.',
                          'parents': ['CHEBI:25408', 'CHEBI:36130']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 5396,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9818082590503912}


"""
Classifies: CHEBI:25634 octadecenoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_octadecenoic_acid(smiles: str):
    """
    Determines if a molecule is an octadecenoic acid (C18 monounsaturated fatty acid).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an octadecenoic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_acid_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_acid_pattern):
        return False, "No carboxylic acid group found"

    # Count carbon atoms
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 18:
        return False, f"Carbon count is {carbon_count}, should be 18"

    # Count double bonds using SMARTS pattern
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    matches = mol.GetSubstructMatches(double_bond_pattern)
    double_bond_count = len(matches)

    if double_bond_count != 1:
        return False, f"Number of double bonds is {double_bond_count}, should be 1"

    # Check if the double bond is in the aliphatic chain
    for match in matches:
        atoms = [mol.GetAtomWithIdx(i) for i in match]
        # Check if both carbons in the double bond are aliphatic
        if all(not atom.GetIsAromatic() for atom in atoms):
            # Find position of double bond by counting carbons from acid end
            acid_carbon = mol.GetSubstructMatch(carboxylic_acid_pattern)[0]
            path_to_double_bond = Chem.GetShortestPath(mol, acid_carbon, match[0])
            position = len(path_to_double_bond) - 1
            
            return True, f"C18 monounsaturated fatty acid with double bond at position {position}"

    return False, "Double bond not found in aliphatic chain"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25634',
                          'name': 'octadecenoic acid',
                          'definition': 'Any member of the group of C18 '
                                        'monounsaturated fatty acids with the '
                                        'double bond located at any position '
                                        'in the chain.',
                          'parents': [   'CHEBI:140948',
                                         'CHEBI:15904',
                                         'CHEBI:25413',
                                         'CHEBI:53339']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.rdMolDescriptors' has no "
               "attribute 'CalcNumAliphaticDoubleBonds'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 48270,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9979326883320929}


import pprint
from distutils.command.config import config
from pathlib import Path
from typing import Union, List, Optional

import pandas as pd
import yaml
from pydantic import BaseModel

from c3p.datamodel import Config, Result, EvaluationResult, EvaluationExperiment
from c3p.generator import safe_name
from c3p.stats import calculate_metrics_pandas


def object_to_header(obj: Union[BaseModel, dict]):
    obj_yaml = yaml.dump(obj.model_dump() if isinstance(obj, BaseModel) else obj)
    obj_yaml = "\n".join([f"# {line}" for line in obj_yaml.split("\n")])
    return obj_yaml


def result_prog_path(r: Result, results_dir: Path):
    cn = safe_name(r.chemical_class.name)
    prog_dir = results_dir / "programs"
    prog_dir.mkdir(exist_ok=True, parents=True)
    return f"{prog_dir / cn}.py"

def write_program(result: Union[Result, EvaluationResult], result_dir: Path, config_yaml: Union[str, Config]):
    """
    Write a program to disk.

    Args:
        r:
        result_dir:
        config_yaml:

    Returns:
    """
    #if isinstance(config_yaml, Config):
    #    config_yaml = object_to_header(config_yaml)
    if isinstance(result, EvaluationResult):
        r = result.train_results.best_result
    prog_path = result_prog_path(r, result_dir)
    with open(prog_path, "w") as f:
        f.write(f'"""\nClassifies: {r.chemical_class.id} {r.chemical_class.name}\n"""')
        f.write("\n")
        f.write(r.code)
        f.write("\n\n\n__metadata__ = ")
        #f.write("\n\n# Classifier Metadata\n# ---\n")
        r_obj = r.model_dump()
        del r_obj["code"]
        cc = r_obj["chemical_class"]
        del cc["instances"]
        del cc["negative_instances"]
        del r_obj["true_positives"]
        del r_obj["false_positives"]
        del r_obj["true_negatives"]
        del r_obj["false_negatives"]
        pp = pprint.PrettyPrinter(indent=4, sort_dicts=False)
        f.write(pp.pformat(r_obj))
        #block = object_to_header(r_obj)
        #f.write(block)

        # f.write(f"\n# Attempt={r.attempt}")
        # f.write(f"\n# Pr={r.precision}")
        # f.write(f"\n# Recall={r.recall}")
        # f.write(f"\n# F1={r.f1}")
        # if isinstance(result, EvaluationResult):
        #     f.write(f"\n# Eval Pr={result.test_result.precision}")
        #     f.write(f"\n# Eval Recall={result.test_result.recall}")
        #     f.write(f"\n# Eval F1={result.test_result.f1}")

def write_programs(results: List[Union[Result, EvaluationResult]], result_dir: Path, config_yaml: Union[str, Config]):
    for r in results:
        write_program(r, result_dir, config_yaml)

def result_as_dict(r: Result, e: EvaluationResult) -> dict:
    d = r.model_dump()
    del d["code"]
    d["chemical_class"] = r.chemical_class.name
    del d["config"]
    #d["attempt"] = e.train_results.best_result.attempt
    d["num_instances"] = len(r.chemical_class.instances)
    for k in ["f1", "precision", "accuracy", "attempt"]:
        d[f"train_{k}"] = getattr(e.train_results.best_result, k)
    return d

def write_eval_results(expt: EvaluationExperiment, results_dir: Path, f1_threshold: Optional[float]=None) -> pd.DataFrame:
    """
    Write the results of an evaluation experiment to disk.

    Args:
        expt:
        results_dir:

    Returns: dataframe of filtered results

    """
    if f1_threshold is None:
        f1_threshold = expt.config.f1_threshold
    expt_path = results_dir / "experiment.yaml"
    with open(expt_path, "w") as f:
        f.write(yaml.dump(expt.model_dump()))
    expt_path_json = results_dir / "experiment.json"
    with open(expt_path_json, "w") as f:
        f.write(expt.model_dump_json(indent=2))
    eval_results = expt.evaluation_results
    for r in eval_results:
        write_program(r, results_dir, expt.config)
    config = expt.config

    df_filtered = pd.DataFrame(
        [result_as_dict(r.test_result, r) for r in eval_results if r.train_results.best_result.f1 > f1_threshold])
    df_filtered["num_test_instances"] = df_filtered["num_true_positives"] + df_filtered["num_false_negatives"]
    df_filtered = df_filtered[df_filtered["num_test_instances"] > 0]
    df_filtered.to_csv(results_dir / "filtered_results.csv", index=False)
    summary_stats = calculate_metrics_pandas(df_filtered)
    summary_stats.to_csv(results_dir / "outcomes_macro.csv", header=False)
    summary_stats = df_filtered.select_dtypes(include=['int64', 'float64']).mean()
    summary_stats.to_csv(results_dir / "outcomes_micro.csv", header=False)
    desc = df_filtered.describe().transpose()
    desc.to_csv(results_dir / "summary_stats_desc.csv")
    return df_filtered





"""
Classifies: CHEBI:24399 glycosaminyl group
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_glycosaminyl_group(smiles: str):
    """
    Determines if a molecule is a glycosaminyl group.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycosaminyl group, False otherwise
        str: Reason for classification
    """
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for key structural features:
    # 1. Must have at least one N-acetyl group (-NC(=O)C)
    # 2. Must have multiple sugar rings
    # 3. Must have a * indicating attachment point
    # 4. Must have multiple OH groups
    
    # Check for * attachment point
    if '*' not in smiles:
        return False, "No attachment point (*) found"
        
    # Count N-acetyl groups
    nacetyl_pattern = Chem.MolFromSmarts('[NH][C](=O)[CH3]')
    if not mol.HasSubstructMatch(nacetyl_pattern):
        return False, "No N-acetyl group found"
        
    # Count rings
    ring_count = rdMolDescriptors.CalcNumRings(mol)
    if ring_count < 1:
        return False, "No sugar rings found"
        
    # Count OH groups
    oh_pattern = Chem.MolFromSmarts('[OH]')
    oh_count = len(mol.GetSubstructMatches(oh_pattern))
    if oh_count < 3:
        return False, "Too few hydroxyl groups"
        
    # Check for pyranose rings (6-membered sugar rings)
    pyranose_pattern = Chem.MolFromSmarts('[C]1[O][C]([C,H])[C][C][C]1')
    if not mol.HasSubstructMatch(pyranose_pattern):
        return False, "No pyranose rings found"

    # If all checks pass, this is likely a glycosaminyl group
    return True, f"Contains {ring_count} sugar rings, N-acetyl group(s), and {oh_count} OH groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24399',
                          'name': 'glycosaminyl group',
                          'definition': 'A glycosyl group obtained by removing '
                                        'the hydroxy group from the hemiacetal '
                                        'function of a glycosamine and, by '
                                        'extension, of a lower oligosaccharide '
                                        'having a glycosamine at the reducing '
                                        'end.',
                          'parents': ['CHEBI:24403']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 10,
    'num_false_positives': 100,
    'num_true_negatives': 12336,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.09090909090909091,
    'recall': 0.9090909090909091,
    'f1': 0.1652892561983471,
    'accuracy': 0.9918855949224713}


"""
Classifies: CHEBI:24317 L-glutamine derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdFMCS

def is_L_glutamine_derivative(smiles: str):
    """
    Determines if a molecule is an L-glutamine derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an L-glutamine derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # L-glutamine core structure (without stereochemistry)
    l_gln = Chem.MolFromSmiles("NC(CCC(=O)N)C(=O)O")
    
    if l_gln is None:
        return None, "Error creating reference structure"

    # Find maximum common substructure
    mcs = rdFMCS.FindMCS([mol, l_gln], 
                        atomCompare=rdFMCS.AtomCompare.CompareElements,
                        bondCompare=rdFMCS.BondCompare.CompareOrder,
                        matchValences=True,
                        ringMatchesRingOnly=True,
                        completeRingsOnly=False,
                        timeout=1)
    
    if mcs.numAtoms < 5:  # Minimum atoms to consider it a derivative
        return False, "Structure too different from L-glutamine core"

    # Check for alpha carbon stereochemistry if present
    alpha_c_pattern = Chem.MolFromSmiles("N[C@H](CCC)C")
    if alpha_c_pattern is not None:
        matches = mol.GetSubstructMatches(alpha_c_pattern)
        if matches:
            # Check if the stereochemistry is correct (S/L configuration)
            chiral_centers = Chem.FindMolChiralCenters(mol)
            for match in matches:
                alpha_c_idx = match[1]  # Index of the alpha carbon
                for center in chiral_centers:
                    if center[0] == alpha_c_idx and center[1] == 'R':
                        return False, "Incorrect stereochemistry at alpha carbon"

    # Look for key modification patterns
    modifications = []
    
    # Check for N-terminal modifications
    n_term_pattern = Chem.MolFromSmarts("[N;!$(NC=O)]C")
    if not mol.HasSubstructMatch(n_term_pattern):
        modifications.append("N-terminal modified")
        
    # Check for C-terminal modifications
    c_term_pattern = Chem.MolFromSmarts("CC(=O)O")
    if not mol.HasSubstructMatch(c_term_pattern):
        modifications.append("C-terminal modified")
        
    # Check for side chain amide modifications
    side_chain_pattern = Chem.MolFromSmarts("CCC(=O)N")
    if not mol.HasSubstructMatch(side_chain_pattern):
        modifications.append("side chain amide modified")
    
    if not modifications:
        modifications.append("contains heteroatom substitutions")
        
    return True, f"L-glutamine derivative with {' and '.join(modifications)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24317',
                          'name': 'L-glutamine derivative',
                          'definition': 'A proteinogenic amino acid derivative '
                                        'resulting from reaction of '
                                        'L-glutamine at the amino group, the '
                                        'carboxy group, or the carboxamide, or '
                                        'from the replacement of any hydrogen '
                                        'of L-glutamine by a heteroatom.',
                          'parents': [   'CHEBI:70813',
                                         'CHEBI:83811',
                                         'CHEBI:83982']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 184,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.6515679442508711}


"""
Classifies: CHEBI:134394 primary allylic alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_primary_allylic_alcohol(smiles: str):
    """
    Determines if a molecule contains a primary allylic alcohol group.
    A primary allylic alcohol has a carbon atom that:
    1. Is attached to a hydroxy (-OH) group
    2. Is attached to two hydrogens (primary)
    3. Is adjacent to a carbon-carbon double bond (allylic)

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains primary allylic alcohol, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Add explicit hydrogens
    mol = Chem.AddHs(mol)
    
    # Find all OH groups
    pattern = Chem.MolFromSmarts("[CH2][OH]")
    matches = mol.GetSubstructMatches(pattern)
    
    if not matches:
        return False, "No primary alcohol groups found"
        
    # For each OH group, check if carbon is adjacent to C=C
    for match in matches:
        c_atom = mol.GetAtomWithIdx(match[0])
        
        # Check if carbon has exactly 2 hydrogens (primary)
        h_count = sum(1 for n in c_atom.GetNeighbors() if n.GetAtomicNum() == 1)
        if h_count != 2:
            continue
            
        # Check neighbors for double bond
        for neighbor in c_atom.GetNeighbors():
            if neighbor.GetAtomicNum() == 6:  # Carbon
                for bond in neighbor.GetBonds():
                    if bond.GetBondType() == Chem.rdchem.BondType.DOUBLE:
                        other_atom = bond.GetOtherAtom(neighbor)
                        if other_atom.GetAtomicNum() == 6:  # C=C
                            return True, "Contains primary allylic alcohol group"
                            
    return False, "No primary allylic alcohol groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134394',
                          'name': 'primary allylic alcohol',
                          'definition': 'An allylic alcohol in which the '
                                        'carbon atom that links the double '
                                        'bond to the hydroxy group is also '
                                        'attached to two hydrogens.',
                          'parents': ['CHEBI:134361', 'CHEBI:15734']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 11068,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9910498523225634}


"""
Classifies: CHEBI:28965 dicarboxylic acid dianion
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_dicarboxylic_acid_dianion(smiles: str):
    """
    Determines if a molecule is a dicarboxylic acid dianion.

    A dicarboxylic acid dianion is a carboxylic acid dianion obtained by deprotonation
    of both carboxy groups of any dicarboxylic acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a dicarboxylic acid dianion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count the number of carboxylate groups
    carboxylates = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetFormalCharge() == -1:
            carboxylates += 1

    if carboxylates != 2:
        return False, "The molecule does not have exactly two carboxylate groups"

    # Check if the carboxylate groups are part of a dicarboxylic acid
    for bond in mol.GetBonds():
        a1 = bond.GetBeginAtom()
        a2 = bond.GetEndAtom()
        if a1.GetSymbol() == 'C' and a2.GetSymbol() == 'C':
            neighbors_a1 = [n.GetSymbol() for n in a1.GetNeighbors()]
            neighbors_a2 = [n.GetSymbol() for n in a2.GetNeighbors()]
            if 'O' in neighbors_a1 and 'O' in neighbors_a2:
                # Found a dicarboxylic acid
                return True, "The molecule is a dicarboxylic acid dianion"

    return False, "The molecule does not contain a dicarboxylic acid group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28965',
                          'name': 'dicarboxylic acid dianion',
                          'definition': 'A carboxylic acid dianion obtained by '
                                        'deprotonation of both carboxy groups '
                                        'of any dicarboxylic acid.',
                          'parents': ['CHEBI:35693', 'CHEBI:38716']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 16,
    'num_false_positives': 100,
    'num_true_negatives': 10435,
    'num_false_negatives': 22,
    'num_negatives': None,
    'precision': 0.13793103448275862,
    'recall': 0.42105263157894735,
    'f1': 0.20779220779220778,
    'accuracy': 0.9884611746902487}


"""
Classifies: CHEBI:131862 HPODE(1-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_HPODE_1__(smiles: str):
    """
    Determines if a molecule is a HPODE(1-) anion.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a HPODE(1-) anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of 18 carbon atoms
    if mol.GetNumHeavyAtoms() != 18:
        return False, "The molecule does not contain 18 heavy atoms"

    # Check for the presence of 2 double bonds
    bond_types = [bond.GetBondType() for bond in mol.GetBonds()]
    if bond_types.count(Chem.BondType.DOUBLE) != 2:
        return False, "The molecule does not contain 2 double bonds"

    # Check for the presence of a carboxylate group
    carboxylate_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetFormalCharge() == -1:
            neighbors = [mol.GetAtomWithIdx(neighbor.GetIdx()) for neighbor in atom.GetNeighbors()]
            if len(neighbors) == 1 and neighbors[0].GetSymbol() == 'C' and neighbors[0].GetFormalCharge() == 0:
                carboxylate_found = True
                break

    if not carboxylate_found:
        return False, "The molecule does not contain a carboxylate group"

    # Check for the presence of a hydroperoxy group
    hydroperoxy_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetFormalCharge() == 0:
            neighbors = [mol.GetAtomWithIdx(neighbor.GetIdx()) for neighbor in atom.GetNeighbors()]
            if len(neighbors) == 2 and neighbors[0].GetSymbol() == 'O' and neighbors[1].GetSymbol() == 'C':
                hydroperoxy_found = True
                break

    if not hydroperoxy_found:
        return False, "The molecule does not contain a hydroperoxy group"

    return True, "The molecule is a HPODE(1-) anion"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131862',
                          'name': 'HPODE(1-)',
                          'definition': 'An octadecanoid anion anion obtained '
                                        'by the deprotonation of the carboxy '
                                        'group of any '
                                        'hydroperoxyoctadecadienoic acid.',
                          'parents': ['CHEBI:131860', 'CHEBI:134019']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183924,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630012234}


"""
Classifies: CHEBI:33692 hydrides
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_hydrides(smiles: str):
    """
    Determines if a molecule is a hydride (a chemical compound of hydrogen with other chemical elements).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydride, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for presence of hydrogen atoms
    num_hydrogen_atoms = sum(atom.GetAtomicNum() == 1 for atom in mol.GetAtoms())
    if num_hydrogen_atoms == 0:
        return False, "No hydrogen atoms present"

    # Check for presence of other elements
    other_elements = set(atom.GetSymbol() for atom in mol.GetAtoms() if atom.GetAtomicNum() != 1)
    if not other_elements:
        return False, "No other elements present besides hydrogen"

    return True, f"Hydride containing hydrogen and elements: {', '.join(other_elements)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33692',
                          'name': 'hydrides',
                          'definition': 'Hydrides are chemical compounds of '
                                        'hydrogen with other chemical '
                                        'elements.',
                          'parents': ['CHEBI:33608', 'CHEBI:37577']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 100,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.0}


"""
Classifies: CHEBI:176910 fatty acyl-L-carnitine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import IPythonConsole
from rdkit.Chem import rdMolDescriptors

def is_fatty_acyl_L_carnitine(smiles: str):
    """
    Determines if a molecule is a fatty acyl-L-carnitine.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a fatty acyl-L-carnitine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carnitine core structure with L configuration
    carnitine_pattern = Chem.MolFromSmarts('[C@H,C@@H](CC([O-])=O)C[N+](C)(C)C')
    if not mol.HasSubstructMatch(carnitine_pattern):
        return False, "Missing L-carnitine core structure"

    # Check for ester linkage
    ester_pattern = Chem.MolFromSmarts('[C@H,C@@H](CC([O-])=O)OC(=O)')
    if not mol.HasSubstructMatch(ester_pattern):
        return False, "Missing ester linkage"

    # Find the acyl group
    acyl_pattern = Chem.MolFromSmarts('C(=O)C')
    matches = mol.GetSubstructMatches(acyl_pattern)
    
    if matches:
        carbon_count = 0
        visited = set()
        
        def count_carbons(atom_idx, mol, visited):
            if atom_idx in visited:
                return 0
            visited.add(atom_idx)
            atom = mol.GetAtomWithIdx(atom_idx)
            if atom.GetSymbol() != 'C':
                return 0
            count = 1
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'C':
                    count += count_carbons(neighbor.GetIdx(), mol, visited)
            return count

        # Start counting from the carbonyl carbon
        for match in matches:
            visited.clear()
            start_idx = match[1]  # Get the carbon after C=O
            current_count = count_carbons(start_idx, mol, visited)
            carbon_count = max(carbon_count, current_count)

        # Check for unsaturation
        double_bond_pattern = Chem.MolFromSmarts('CC=CC')
        has_double_bonds = mol.HasSubstructMatch(double_bond_pattern)
        
        fatty_acid_type = ""
        if has_double_bonds:
            fatty_acid_type = "unsaturated"
        else:
            fatty_acid_type = "saturated"

        # Any acyl chain can be considered a fatty acid for this class
        return True, f"Valid fatty acyl-L-carnitine with {fatty_acid_type} fatty acid ({carbon_count} carbons)"

    return False, "Missing fatty acid component"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:176910',
                          'name': 'fatty acyl-L-carnitine',
                          'definition': 'Any O-acylcarnitine in which the '
                                        'carnitine component has '
                                        'L-configuration and the acyl group is '
                                        'a fatty acid.',
                          'parents': ['CHEBI:61697', 'CHEBI:75659']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('CCCCCCCCCCCCCCCC(=O)O[C@H](CC([O-])=O)C[N+](C)(C)C', 'Acyl "
               "chain too short to be considered a fatty acid'), "
               "('C[N+](C)(C)C[C@@H](CC([O-])=O)OC([*])=O', 'Acyl chain too "
               "short to be considered a fatty acid'), "
               "('C[N+](C)(C)C[C@@H](CC([O-])=O)OC([*])=O', 'Acyl chain too "
               "short to be considered a fatty acid'), "
               "('O([C@@H](C[N+](C([2H])([2H])[2H])(C)C)CC([O-])=O)C(=O)CC', "
               "'Acyl chain too short to be considered a fatty acid'), "
               "('C[N+](C)(C)C[C@@H](CC([O-])=O)OC([*])=O', 'Acyl chain too "
               "short to be considered a fatty acid'), "
               "('[C@H](OC(*)=O)(C[N+](C([2H])([2H])[2H])(C)C)CC(=O)[O-]', "
               "'Acyl chain too short to be considered a fatty acid'), "
               "('CCCC\\\\C=C/CCCCCCCC(=O)O[C@H](CC([O-])=O)C[N+](C)(C)C', "
               "'Acyl chain too short to be considered a fatty acid')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 98915,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.9989901234069197}


"""
Classifies: CHEBI:25940 peroxides
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_peroxides(smiles: str):
    """
    Determines if a molecule contains a peroxide group (ROOR').
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains peroxide group, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Find all oxygen atoms
    oxygen_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'O']
    
    # Check for O-O bonds
    for o1 in oxygen_atoms:
        for neighbor in o1.GetNeighbors():
            if neighbor.GetSymbol() == 'O':
                o2 = neighbor
                
                # Check that both oxygens have other neighbors (R groups)
                o1_neighbors = [n for n in o1.GetNeighbors() if n.GetIdx() != o2.GetIdx()]
                o2_neighbors = [n for n in o2.GetNeighbors() if n.GetIdx() != o1.GetIdx()]
                
                if len(o1_neighbors) > 0 and len(o2_neighbors) > 0:
                    # Get R group details
                    r1 = o1_neighbors[0].GetSymbol()
                    r2 = o2_neighbors[0].GetSymbol()
                    return True, f"Contains peroxide group with R groups: {r1} and {r2}"
                    
    return False, "No peroxide group (ROOR') found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25940',
                          'name': 'peroxides',
                          'definition': "Compounds of structure ROOR'.",
                          'parents': ['CHEBI:25741']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 89177,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9988799534060617}


"""
Classifies: CHEBI:167057 monolysocardiolipin(2-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_monolysocardiolipin_2__(smiles: str):
    """
    Determines if a molecule is a monolysocardiolipin(2-), defined as 'The organophosphate oxoanion formed 
    from any monolysocardiolipin by deprotonation of the two phospho groups; principal species present at pH 7.3'.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a monolysocardiolipin(2-), False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of two phosphate groups
    phosphate_count = sum(atom.GetSymbol() == 'P' for atom in mol.GetAtoms())
    if phosphate_count != 2:
        return False, "Does not contain exactly two phosphate groups"

    # Check for the presence of two deprotonated phosphate groups
    deprotonated_phosphate_count = sum(atom.GetFormalCharge() == -1 for atom in mol.GetAtoms())
    if deprotonated_phosphate_count != 2:
        return False, "Does not contain exactly two deprotonated phosphate groups"

    # Check for the presence of a glycerol backbone
    glycerol_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'C' and atom.GetDegree() == 3 and atom.GetTotalNumHs() == 1]
    if len(glycerol_atoms) != 2:
        return False, "Does not contain a glycerol backbone"

    # Check for the presence of a fatty acid chain
    fatty_acid_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'C' and atom.GetDegree() == 1 and atom.GetTotalNumHs() == 3]
    if not fatty_acid_atoms:
        return False, "Does not contain a fatty acid chain"

    return True, "Molecule is a monolysocardiolipin(2-)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:167057',
                          'name': 'monolysocardiolipin(2-)',
                          'definition': 'The organophosphate oxoanion formed '
                                        'from any monolysocardiolipin by '
                                        'deprotonation of the two phospho '
                                        'groups; principal species present at '
                                        'pH 7.3',
                          'parents': ['CHEBI:76529']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183921,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.999994562912539}


"""
Classifies: CHEBI:23115 chlorohydrocarbon
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_chlorohydrocarbon(smiles: str):
    """
    Determines if a molecule is a chlorohydrocarbon (compound derived from hydrocarbon 
    by replacing at least one hydrogen with chlorine).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a chlorohydrocarbon, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count atoms
    num_carbons = 0
    num_chlorines = 0
    num_other = 0
    other_atoms = set()
    
    for atom in mol.GetAtoms():
        symbol = atom.GetSymbol()
        if symbol == 'C':
            num_carbons += 1
        elif symbol == 'Cl':
            num_chlorines += 1
        else:
            num_other += 1
            other_atoms.add(symbol)

    # Must have at least one carbon and one chlorine
    if num_carbons == 0:
        return False, "No carbon atoms found"
    if num_chlorines == 0:
        return False, "No chlorine atoms found"

    # Create parent hydrocarbon by replacing chlorines with hydrogens
    parent_smiles = smiles.replace('Cl','H')
    parent_mol = Chem.MolFromSmiles(parent_smiles)
    
    if parent_mol is None:
        return False, "Could not generate valid parent hydrocarbon structure"

    # Check if parent molecule is a hydrocarbon
    for atom in parent_mol.GetAtoms():
        if atom.GetSymbol() not in ['C', 'H']:
            return False, f"Contains non-hydrocarbon atoms: {', '.join(other_atoms)}"

    return True, f"Chlorohydrocarbon with {num_carbons} carbons and {num_chlorines} chlorines" + \
           (f" (also contains {', '.join(other_atoms)})" if other_atoms else "")


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23115',
                          'name': 'chlorohydrocarbon',
                          'definition': 'A compound derived from a hydrocarbon '
                                        'by replacing at least one hydrogen '
                                        'atom with a chlorine atom.',
                          'parents': ['CHEBI:24472', 'CHEBI:36683']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183854,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999564891059599}


"""
Classifies: CHEBI:23003 carbamate ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_carbamate_ester(smiles: str):
    """
    Determines if a molecule contains a carbamate ester group (R-O-C(=O)-N).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains carbamate ester, False otherwise
        str: Reason for classification
    """
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for carbamate ester: R-O-C(=O)-N
    carbamate_pattern = Chem.MolFromSmarts('[#6,#1]-[O;X2]-[C;X3](=[O;X1])-[N;X3]')
    
    matches = mol.GetSubstructMatches(carbamate_pattern)
    
    if not matches:
        return False, "No carbamate ester group found"
        
    # Get details about the carbamate group(s)
    carbamate_details = []
    for match in matches:
        # Get the nitrogen atom
        n_atom = mol.GetAtomWithIdx(match[3])
        
        # Check substituents on nitrogen
        n_substituents = []
        for neighbor in n_atom.GetNeighbors():
            if neighbor.GetIdx() != match[2]:  # Skip the carbonyl carbon
                if neighbor.GetSymbol() == 'H':
                    n_substituents.append('H')
                else:
                    n_substituents.append('R')
                    
        if len(n_substituents) == 0:
            n_desc = "unsubstituted"
        elif len(n_substituents) == 1:
            n_desc = "mono-substituted"
        else:
            n_desc = "di-substituted"
            
        carbamate_details.append(f"{n_desc} nitrogen")
    
    details_str = ", ".join(set(carbamate_details))
    return True, f"Contains carbamate ester group(s) with {details_str}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23003',
                          'name': 'carbamate ester',
                          'definition': 'Any ester of carbamic acid or its '
                                        'N-substituted derivatives.',
                          'parents': ['CHEBI:33308']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 28,
    'num_false_positives': 100,
    'num_true_negatives': 15614,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.21875,
    'recall': 0.9655172413793104,
    'f1': 0.35668789808917195,
    'accuracy': 0.993584450231849}


"""
Classifies: CHEBI:133131 spiro-epoxide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_spiro_epoxide(smiles: str):
    """
    Determines if a molecule contains a spiro-epoxide motif - where a carbon atom
    of an epoxide ring is the only common member of two rings.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains spiro-epoxide, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Get ring information
    rings = mol.GetRingInfo()
    if not rings.NumRings():
        return False, "No rings found"

    # Find epoxide rings (3-membered rings containing oxygen)
    epoxide_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 3:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            atom_symbols = [atom.GetSymbol() for atom in atoms]
            if 'O' in atom_symbols:
                epoxide_rings.append(ring)

    if not epoxide_rings:
        return False, "No epoxide rings found"

    # Check each epoxide ring for spiro fusion
    for epoxide_ring in epoxide_rings:
        # Get carbon atoms in epoxide ring
        epoxide_carbons = [i for i in epoxide_ring if mol.GetAtomWithIdx(i).GetSymbol() == 'C']
        
        for carbon_idx in epoxide_carbons:
            # Find all rings containing this carbon
            rings_with_carbon = []
            for ring in rings.AtomRings():
                if carbon_idx in ring:
                    rings_with_carbon.append(set(ring))
            
            # Check if this carbon is a spiro center
            if len(rings_with_carbon) == 2:
                # Check if this carbon is the only common atom between the rings
                intersection = rings_with_carbon[0].intersection(rings_with_carbon[1])
                if len(intersection) == 1 and carbon_idx in intersection:
                    return True, f"Spiro-epoxide found with spiro carbon at index {carbon_idx}"

    return False, "No spiro-epoxide pattern found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:133131',
                          'name': 'spiro-epoxide',
                          'definition': 'An oxaspiro compound in which a '
                                        'carbon atom of an epoxide ring is the '
                                        'only common member of two rings.',
                          'parents': ['CHEBI:32955', 'CHEBI:37948']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 89404,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9988827939089924}


"""
Classifies: CHEBI:24397 glycophospholipid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_glycophospholipid(smiles: str):
    """
    Determines if a molecule is a glycophospholipid based on presence of both
    phosphate and carbohydrate structural components.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycophospholipid, False otherwise
        str: Reason for classification
    """
    
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"
            
        # Check for phosphate group
        has_phosphate = False
        matches = mol.GetSubstructMatches(Chem.MolFromSmarts('[P](=O)([O,OH])[O,OH]'))
        if len(matches) > 0:
            has_phosphate = True
            
        if not has_phosphate:
            return False, "No phosphate group found"
            
        # Check for carbohydrate components
        # Look for pyranose rings with multiple OH groups
        has_carb = False
        
        # Pattern for pyranose ring with multiple OH groups
        pyranose_pattern = Chem.MolFromSmarts('[C]1[C][C]([OH1,OH0])[C]([OH1,OH0])[C]([OH1,OH0])[C]1[OH1,OH0]')
        if mol.HasSubstructMatch(pyranose_pattern):
            has_carb = True
            
        # Alternative pattern for sugar rings
        sugar_pattern = Chem.MolFromSmarts('[C]1[O][C]([CH2][OH1,OH0])[C]([OH1,OH0])[C]([OH1,OH0])[C]1[OH1,OH0]')
        if mol.HasSubstructMatch(sugar_pattern):
            has_carb = True
            
        if not has_carb:
            return False, "No carbohydrate component found"
            
        # Count OH groups
        oh_count = len(mol.GetSubstructMatches(Chem.MolFromSmarts('[OH1]')))
        
        # Additional checks for lipid characteristics
        carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
        
        if has_phosphate and has_carb and oh_count >= 3 and carbon_count > 10:
            return True, "Contains both phosphate and carbohydrate components with appropriate lipid characteristics"
            
        return False, "Missing required structural components"
        
    except Exception as e:
        return None, f"Error processing molecule: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24397',
                          'name': 'glycophospholipid',
                          'definition': 'Any phospholipid that contain both '
                                        'phosphate and carbohydrate as '
                                        'integral structural components.',
                          'parents': ['CHEBI:16247', 'CHEBI:33563']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 12,
    'num_false_positives': 100,
    'num_true_negatives': 16050,
    'num_false_negatives': 9,
    'num_negatives': None,
    'precision': 0.10714285714285714,
    'recall': 0.5714285714285714,
    'f1': 0.18045112781954886,
    'accuracy': 0.9932595386803538}


"""
Classifies: CHEBI:26605 saponin
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_saponin(smiles: str):
    """
    Determines if a molecule is a saponin based on structural characteristics.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a saponin, False otherwise
        str: Reason for classification
    """
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"
            
        # Check for minimum size - saponins are large molecules
        num_atoms = mol.GetNumAtoms()
        if num_atoms < 40:
            return False, "Molecule too small to be a saponin"
            
        # Check for presence of sugar moiety (glycoside)
        sugar_pattern = Chem.MolFromSmarts('[CH]1[CH][CH][CH][CH]O1') # Basic pyranose ring
        if not mol.HasSubstructMatch(sugar_pattern):
            return False, "No sugar/glycoside moiety found"
            
        # Check for steroid/triterpenoid core
        # Look for multiple 6-membered rings characteristic of steroid/triterpenoid skeleton
        rings = mol.GetRingInfo()
        six_mem_rings = sum(1 for ring in rings.AtomRings() if len(ring) == 6)
        if six_mem_rings < 4:
            return False, "No steroid/triterpenoid core structure found"
            
        # Check for characteristic O-glycosidic linkage
        glycosidic_pattern = Chem.MolFromSmarts('[CH]O[CH]1[CH][CH][CH][CH]O1')
        if not mol.HasSubstructMatch(glycosidic_pattern):
            return False, "No O-glycosidic linkage found"
            
        # Calculate lipophilicity using logP
        logp = Descriptors.MolLogP(mol)
        if logp < 0:
            return False, "Molecule not sufficiently lipophilic"
            
        # Molecule has passed all saponin criteria
        sugar_count = len(mol.GetSubstructMatches(sugar_pattern))
        return True, f"Saponin with {sugar_count} sugar moieties detected"
        
    except:
        return None, None


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26605',
                          'name': 'saponin',
                          'definition': 'A glycoside that is a compound '
                                        'containing one or more hydrophilic '
                                        'glycoside moieties combined with a '
                                        'lipophilic triterpenoid or steroid '
                                        'derivative. Found in particular '
                                        'abundance in plant species.',
                          'parents': ['CHEBI:24400']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 39,
    'num_false_positives': 100,
    'num_true_negatives': 39917,
    'num_false_negatives': 43,
    'num_negatives': None,
    'precision': 0.2805755395683453,
    'recall': 0.47560975609756095,
    'f1': 0.35294117647058815,
    'accuracy': 0.9964338262799571}


"""
Classifies: CHEBI:27532 L-cysteine thioether
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_L_cysteine_thioether(smiles: str):
    """
    Determines if a molecule is an L-cysteine thioether.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an L-cysteine thioether, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of sulfur atom(s)
    if not any(atom.GetSymbol() == 'S' for atom in mol.GetAtoms()):
        return False, "No sulfur atom found"

    # Check for the presence of carboxyl group (-COOH)
    has_carboxyl = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            neighbors = [mol.GetAtomWithIdx(n).GetSymbol() for n in atom.GetNeighbors()]
            if 'O' in neighbors and 'O' in neighbors:
                has_carboxyl = True
                break

    if not has_carboxyl:
        return False, "No carboxyl group found"

    # Check for the presence of amino group (-NH2)
    has_amino = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'N':
            neighbors = [mol.GetAtomWithIdx(n).GetSymbol() for n in atom.GetNeighbors()]
            if 'H' in neighbors and 'H' in neighbors:
                has_amino = True
                break

    if not has_amino:
        return False, "No amino group found"

    # Check if the sulfur atom is bonded to an aromatic ring or an aliphatic chain
    is_thioether = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'S':
            neighbors = [mol.GetAtomWithIdx(n) for n in atom.GetNeighbors()]
            for neighbor in neighbors:
                if neighbor.GetIsAromatic():
                    is_thioether = True
                    break
                elif neighbor.GetHybridization() == Chem.HybridizationType.SP3:
                    is_thioether = True
                    break

    if is_thioether:
        return True, "L-cysteine thioether"
    else:
        return False, "Not an L-cysteine thioether"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27532',
                          'name': 'L-cysteine thioether',
                          'definition': 'Any L-cysteine derivative obtained by '
                                        'conversion of the thiol group into a '
                                        'sulfide.',
                          'parents': ['CHEBI:16385', 'CHEBI:47910']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.GetAtomWithIdx(Mol, Atom)\n'
             'did not match C++ signature:\n'
             '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int idx)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:27013 tocopherol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_tocopherol(smiles: str):
    """
    Determines if a molecule is a tocopherol or tocotrienol.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a tocopherol or tocotrienol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the chroman-6-ol nucleus
    match = mol.GetSubstructMatches(Chem.MolFromSmarts('c1c(O)c2OCCc2cc1'))
    if not match:
        return False, "Chroman-6-ol nucleus not found"

    # Check for methyl group at position 2
    chroman_ring = mol.GetAtomWithIdx(match[0][1])
    if chroman_ring.GetNumExplicitHs() != 1:
        return False, "No methyl group at position 2"

    # Check for saturated hydrocarbon chain with three isoprenoid units
    chain_atoms = [a for a in mol.GetAtoms() if a.GetIdx() not in match[0]]
    chain_length = sum(1 for a in chain_atoms if a.GetHybridization() == Chem.HybridizationType.SP3)
    if chain_length != 16:
        return False, "Hydrocarbon chain does not have 16 carbon atoms"

    double_bonds = sum(1 for a in chain_atoms if a.GetHybridization() == Chem.HybridizationType.SP2)
    if double_bonds == 0:
        return True, "Tocopherol"
    elif double_bonds == 3:
        return True, "Tocotrienol"
    else:
        return False, "Hydrocarbon chain does not match tocopherol or tocotrienol"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27013',
                          'name': 'tocopherol',
                          'definition': 'A collective name for a group of '
                                        'closely related lipids that contain a '
                                        'chroman-6-ol nucleus substituted at '
                                        'position 2 by a methyl group and by a '
                                        'saturated hydrocarbon chain '
                                        'consisting of three isoprenoid units. '
                                        'They are designated as alpha-, beta-, '
                                        'gamma-, and delta-tocopherol '
                                        'depending on the number and position '
                                        'of additional methyl substituents on '
                                        'the aromatic ring. Tocopherols occur '
                                        'in vegetable oils and vegetable oil '
                                        'products, almost exclusively with '
                                        'R,R,R configuration. Tocotrienols '
                                        'differ from tocopherols only in '
                                        'having three double bonds in the '
                                        'hydrocarbon chain.',
                          'parents': ['CHEBI:39437']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183918,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9999945628238518}


"""
Classifies: CHEBI:18368 D-glucosyl-N-acylsphingosine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_D_glucosyl_N_acylsphingosine(smiles: str):
    """
    Determines if a molecule is a D-glucosyl-N-acylsphingosine (Sphingosine substituted at the 1-hydroxy group by a D-glucosyl group and at the 2-amino group by an acyl group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a D-glucosyl-N-acylsphingosine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the sphingosine backbone
    sphingosine_pattern = Chem.MolFromSmarts('[NH1][CH2][CH][CH][CH][CH][CH][CH][CH][CH][CH][CH][CH][CH][CH][CH][CH][CH]O')
    match = mol.GetSubstructMatches(sphingosine_pattern)
    if not match:
        return False, "Sphingosine backbone not found"

    # Check for D-glucosyl substituent at the 1-hydroxy position
    glucosyl_pattern = Chem.MolFromSmarts('OC[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O')
    match = mol.GetSubstructMatches(glucosyl_pattern)
    if not match:
        return False, "D-glucosyl group not found"

    # Check for acyl substituent at the 2-amino position
    acyl_pattern = Chem.MolFromSmarts('C(=O)')
    match = mol.GetSubstructMatches(acyl_pattern, maxMatches=1)
    if not match:
        return False, "Acyl group not found"

    acyl_atom = mol.GetAtomWithIdx(list(match)[0][0])
    if acyl_atom.GetTotalNumHs() != 0:
        return False, "Acyl group not attached to amino group"

    return True, "Molecule is a D-glucosyl-N-acylsphingosine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18368',
                          'name': 'D-glucosyl-N-acylsphingosine',
                          'definition': 'Sphingosine substituted at the '
                                        '1-hydroxy group by a D-glucosyl group '
                                        'and at the 2-amino group by an acyl '
                                        'group.',
                          'parents': ['CHEBI:36500']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183920,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.999994562882977}